﻿package com
{
	import com.bit101.components.HSlider;
	import com.bit101.components.InputText;
	import com.bit101.components.Label;
	import com.bit101.components.Panel;
	import com.bit101.components.PushButton;
	import com.bit101.components.Slider;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.ui.Mouse;
	
	/**
	 * ...
	 * @author 
	 */
	public class Main extends Sprite 
	{
		private var FPS:int;
		private var r:Sprite;
		private var keyframes:Array;
		private var mouseDown:Boolean;
		private var running:Boolean;
		private var time:Number;
		private var interpolator:IInterpolator;
		private var slider:HSlider;
		private var captureMouse :Boolean ;
		private var frameLabel:Label;
		private var lastFrame:Number;
		
		public function Main():void 
		{
			captureMouse = true;
			time = 0;
			FPS = 30;
			mouseDown = false;
			running = false;
			keyframes = new Array;
			interpolator = new NearestInterpolator();
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}

		private function updateFrame():void {
			var f:Frame = interpolator.get(time/1000);
			r.x = f.x;
			r.y = f.y;
			r.rotation = f.r * 180 / Math.PI;
			slider.value = Math.round((time / 1000) * FPS);
			frameLabel.text =  Math.round((time / 1000) * FPS).toString();
		}
		
		private function onEnterFrame(e:Event = null):void {
			var date:Date = new Date;
			var now:Number = date.getTime();
			if (running) {
				if (time/1000 >= keyframes.length-1) {
					running = false;
					time = 0;
				}
				time += now - lastFrame;
				updateFrame();
			}
			lastFrame = now;
			graphics.clear();
			graphics.lineStyle(1, 0);
			for (var i : int = 0; i < keyframes.length; ++i ) {
				graphics.moveTo(keyframes[i].x, keyframes[i].y);
				graphics.lineTo(keyframes[i].x + Math.cos(keyframes[i].r) * 10, keyframes[i].y + Math.sin(keyframes[i].r) * 10);
			}
		}
		
		private function pause(e:Event):void {
			running = false;
		}
		
		private function run(e:Event = null):void {
			if(keyframes.length > 1) {
				if(time <= 0 || time >= keyframes.length) {
					time = 0;
					lastFrame = (new Date).getTime();
					slider.maximum = FPS * (keyframes.length-1);
				}
				running = true;
				interpolator.setKeyframes(keyframes);
			}
		}
		
		private function onMouseMove(e:MouseEvent):void {
			if(captureMouse && mouseDown && keyframes.length > 0) {
				var lastFrame:Frame = keyframes[keyframes.length - 1];
				var dx:Number = e.stageX - lastFrame.x;
				var dy:Number = e.stageY - lastFrame.y;
				var l:Number = Math.sqrt(dx * dx + dy * dy);
				dx /= l;
				keyframes[keyframes.length - 1].r = Math.acos(dx);
				if (dy < 0) {
					keyframes[keyframes.length - 1].r = 2*Math.PI - keyframes[keyframes.length - 1].r;
				}
			}
		}
		
		private function onMouseUp(e:MouseEvent):void {
			onMouseMove(e);
			mouseDown = false;
		}
		
		private function onMouseDown(e:MouseEvent):void {
			if(captureMouse && e.stageY < 200 && keyframes.length < 10) {
				mouseDown = true;
				var f:Frame = new Frame(e.stageX, e.stageY, 0);
				keyframes.push(f);
				slider.maximum = FPS * (keyframes.length-1);
			}
		}
		
		private function clear(e:Event = null):void {
			keyframes = new Array;
			running = false;
			time = 0;
			slider.maximum = 30;
		}
		
		private function onSlider(e:Event):void {
			interpolator.setKeyframes(keyframes);
			time = 1000*(slider.value/FPS);
			updateFrame();
		}
		
		private function onSetFps(e:Event):void {
			captureMouse = false;
			var p:Panel = new Panel(this, 125, 100);
			p.width = 150;
			var i:InputText = new InputText(p, 25, 20, "Enter FPS");
			var b:PushButton = new PushButton(p, 25, 50, "Set", function():void {
				p.parent.removeChild(p);
				captureMouse = true;
				FPS = parseInt(i.text);
				slider.maximum = FPS * (keyframes.length-1);
			});
		}
		
		private function onNextFrame(e:Event):void {
			if(time/1000 < keyframes.length-1) {
				time += 1000 / FPS;
				updateFrame();
			}
		}

		private function onPrevFrame(e:Event):void {
			if(time > 0) {
				time -= 1000 / FPS;
				updateFrame();
			}
			
		}
		
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			// entry point
			new PushButton(this, 10, 260, "Prev Frame", onPrevFrame);
			new PushButton(this, 120, 260, "Next Frame", onNextFrame);
			new PushButton(this, 230, 260, "Set FPS", onSetFps);
			new PushButton(this, 10, 280, "Run", run);
			new PushButton(this, 120, 280, "Pause", pause);
			new PushButton(this, 230, 280, "Clear", clear);
			
			slider = new HSlider(this, 10, 240, onSlider );
			slider.width = 300;
			slider.minimum = 0;
			slider.maximum = 30;
			frameLabel = new Label(this, 320, 235, "frame"); 
			
			r = new Sprite;
			r.graphics.lineStyle(1, 0);
			r.graphics.moveTo(0, 0);
			r.graphics.lineTo(10, 0);
			r.graphics.drawRect(-5, -5, 10, 10);
			addChild(r);
			r.x = 0;
			r.y = 0;
			r.rotation = 0;
		}
		
	}
	
}